#include "DIALOG.h"
int x1, y1, ang1 = 0;
int x2 = 10;
int y3 = 10;

typedef struct {
  void              (* pfMotionHandler)(void);
  void               * pchCrit;
  GUI_TIMER_HANDLE     hTimer;
  int                  SignX, SignY;
  int                  State;
  I32                  Time0;
  I32                  SpeedX, SpeedY;
  I32                  DecelerationX, DecelerationY;
  I32                  sX, sY;
  I32                  DestX, DestY;
} WM_MOTION_PRIVATE_struct;

static void _InitTagHandler(WM_MOTION_INFO * pInfo) {
  static void (* pfGetMotionTag)(GUI_PID_STATE *, void *, WM_HWIN);
  void       (** ppfGetTag)     (GUI_PID_STATE *, void *, WM_HWIN);

  //
  // Address to the function pointer in the motion context
  //
  ppfGetTag = (void (**)(GUI_PID_STATE *, void *, WM_HWIN))(((U8 *)pInfo) + sizeof(WM_MOTION_INFO) + sizeof(WM_MOTION_PRIVATE_struct));
  //
  // Initially the tag handler is set to XY movements, save it in a variable.
  //
  if (pfGetMotionTag == NULL) {
    pfGetMotionTag = *ppfGetTag;
  }
  //
  // Manually reset the tag handler to XY movements if needed.
  // This is not required for radial movements.
  //
  if ((pInfo->Flags & WM_CF_MOTION_X) || (pInfo->Flags & WM_CF_MOTION_Y)) {
    *ppfGetTag = pfGetMotionTag;
  }
}

void _cbWin1(WM_MESSAGE * pMsg) {
  WM_MOTION_INFO * pInfo;
  switch (pMsg->MsgId) {
  case WM_MOTION:
    pInfo = (WM_MOTION_INFO *)pMsg->Data.p;
    switch (pInfo->Cmd) {
    case WM_MOTION_INIT:
      pInfo->Flags = WM_CF_MOTION_R | WM_MOTION_MANAGE_BY_WINDOW;
      _InitTagHandler(pInfo);
      break;
    case WM_MOTION_MOVE:
      ang1 += pInfo->da;
      WM_InvalidateWindow(pMsg->hWin);
      break;
    case WM_MOTION_GETPOS:
      pInfo->xPos = ang1;
      break;
    }
    break;
  case WM_PAINT:
    GUI_SetBkColor(GUI_WHITE);
    GUI_Clear();
    GUI_SetColor(GUI_BLACK);
    GUI_DispStringHCenterAt("Radial motion", 100, 10);
    x1 = (int)(GUI__SinHQ(ang1 * 100) * 85 * 0.0000152587890625f) + 100;
    y1 = (int)(GUI__CosHQ(ang1 * 100) * 85 * 0.0000152587890625f) + 100;		
    GUI_SetColor(GUI_RED);
    GUI_FillCircle(x1, y1, 10);
    break;
  default:
    WM_DefaultProc(pMsg);
    break;
  }
}
void _cbWin2(WM_MESSAGE * pMsg) {
  WM_MOTION_INFO * pInfo;
  switch (pMsg->MsgId) {
  case WM_MOTION:
    pInfo = (WM_MOTION_INFO *)pMsg->Data.p;
    switch (pInfo->Cmd) {
    case WM_MOTION_INIT:
      pInfo->Flags = WM_CF_MOTION_X | WM_MOTION_MANAGE_BY_WINDOW;
      _InitTagHandler(pInfo);
      break;
    case WM_MOTION_MOVE:
      x2 += pInfo->dx;
      if (x2 < 10)
        x2 = 10;
      if (x2 > 290)
        x2 = 290;
      WM_InvalidateWindow(pMsg->hWin);
      break;
    case WM_MOTION_GETPOS:
      pInfo->xPos = x2;
      break;
    }
    break;
  case WM_PAINT:
    GUI_SetBkColor(GUI_WHITE);
    GUI_Clear();
    GUI_SetColor(GUI_BLACK);
    GUI_DispStringHCenterAt("Horizontal motion", 150, 10);
    GUI_SetColor(GUI_BLUE);
    GUI_FillCircle(x2, 50, 10);
    break;
  default:
    WM_DefaultProc(pMsg);
    break;
  }
}
void _cbWin3(WM_MESSAGE * pMsg) {
  WM_MOTION_INFO * pInfo;
  switch (pMsg->MsgId) {
  case WM_MOTION:
    pInfo = (WM_MOTION_INFO *)pMsg->Data.p;
    switch (pInfo->Cmd) {
    case WM_MOTION_INIT:
      pInfo->Flags = WM_CF_MOTION_Y | WM_MOTION_MANAGE_BY_WINDOW;
      _InitTagHandler(pInfo);
      break;
    case WM_MOTION_MOVE:
      y3 += pInfo->dy;
      if (y3 < 10)
        y3 = 10;
      if (y3 > 290)
        y3 = 290;
      WM_InvalidateWindow(pMsg->hWin);
      break;
    case WM_MOTION_GETPOS:
      pInfo->yPos = y3;
      break;
    }
    break;
  case WM_PAINT:
    GUI_SetBkColor(GUI_WHITE);
    GUI_Clear();
    GUI_SetColor(GUI_BLACK);
    GUI_DispStringHCenterAt("Vertical motion", 50, 10);
    GUI_SetColor(GUI_GREEN);
    GUI_FillCircle(50, y3, 10);
    break;
  default:
    WM_DefaultProc(pMsg);
    break;
  }
}
void MainTask(void) {
  GUI_Init();
  WM_MULTIBUF_Enable(1);
  WM_SetDesktopColor(GUI_BLACK);
  WM_SetScreenSize(1024, 768);
  WM_CreateWindow(100, 100, 200, 200, WM_CF_SHOW, _cbWin1, 0);
  WM_CreateWindow(400, 100, 300, 100, WM_CF_SHOW, _cbWin2, 0);
  WM_CreateWindow(800, 100, 100, 300, WM_CF_SHOW, _cbWin3, 0);
  WM_MOTION_Enable(1);
  while (1) {
    GUI_Delay(50);
  }
}
